@@ -25,7 +25,7 @@ from simplipy.errors import (
 )
 from simplipy.system.v2 import SystemV2
 from simplipy.system.v3 import SystemV3
-from simplipy.util import schedule_callback
+from simplipy.util import execute_callback
 from simplipy.util.auth import get_auth0_code_challenge, get_auth0_code_verifier
 from simplipy.websocket import WebsocketClient
 
@@ -111,9 +111,9 @@ class API:  # pylint: disable=too-many-instance-attributes
 
         :param username: A SimpliSafe account username/email address
         :type username: ``str``
-        :param password: A SimpliSafe account password
-        :type password: ``str``
-        :param session: The ``aiohttp`` ``ClientSession`` session used for all HTTP requests
+        :param pswd: A SimpliSafe account password
+        :type pswd: ``str``
+        :param session: An ``aiohttp`` ``ClientSession``
         :type session: ``aiohttp.client.ClientSession``
         :param request_retries: The default number of request retries to use
         :type request_retries: ``int``
@@ -147,30 +147,38 @@ class API:  # pylint: disable=too-many-instance-attributes
                 },
             )
             auth0_resp.raise_for_status()
-        except ClientResponseError as err:
+        except Exception as err:  # pylint: disable-broad-except
             raise SimplipyError(
                 f"Error while determining the Auth0 login URL: {err}"
             ) from err
 
-        # Attempt to login:
-        login_resp = await session.request(
-            "post",
-            f"{AUTH_URL_BASE}{auth0_resp.headers['Location']}",
-            data={
-                "password": password,
-                "username": username,
-            },
-        )
+        LOGGER.debug("Auth0 response headers: %s", auth0_resp.headers)
 
-        body = await login_resp.text()
-        LOGGER.debug("Login attempt response body: %s", body)
+        if not auth0_resp.headers["Location"].startswith("/"):
+            # Login has already occurred (according to the session cookies we have), so
+            # we already have the login verification URL:
+            api._login_verification_url = URL(auth0_resp.headers["Location"])
+        else:
+            # Attempt to login:
+            login_resp = await session.request(
+                "post",
+                f"{AUTH_URL_BASE}{auth0_resp.headers['Location']}",
+                data={
+                    "password": password,
+                    "username": username,
+                },
+            )
 
-        if "invalid_user_password" in body:
-            raise InvalidCredentialsError("Invalid username/password")
-        if "too-many-sms" in body:
-            raise Verify2FAError("SMS 2FA limit per hour exceeded; try again later")
+            body = await login_resp.text()
+            LOGGER.debug("Login attempt response body: %s", body)
+
+            if "invalid_user_password" in body:
+                raise InvalidCredentialsError("Invalid username/password")
+            if "too-many-sms" in body:
+                raise Verify2FAError("SMS 2FA limit per hour exceeded; try again later")
+
+            api._login_verification_url = login_resp.url
 
-        api._login_verification_url = login_resp.url
         LOGGER.debug("API Verification URL: %s", api._login_verification_url)
         assert api._login_verification_url
 
@@ -197,10 +205,10 @@ class API:  # pylint: disable=too-many-instance-attributes
 
         :param refresh_token: The refresh token
         :type refresh_token: ``str``
-        :param session: The ``aiohttp`` ``ClientSession`` session used for all HTTP requests
+        :param session: An ``aiohttp`` ``ClientSession``
         :type session: ``aiohttp.client.ClientSession``
         :param request_retries: The default number of request retries to use
-        :typ request_retries: ``int``
+        :type request_retries: ``int``
         :rtype: :meth:`simplipy.api.API`
         """
         api = cls(session=session, request_retries=request_retries)
@@ -220,7 +228,7 @@ class API:  # pylint: disable=too-many-instance-attributes
                 allow_redirects=False,
             )
             auth_resume_resp.raise_for_status()
-        except ClientResponseError as err:
+        except Exception as err:  # pylint: disable-broad-except
             raise SimplipyError(
                 f"Error while attempting to get authorization code: {err}"
             ) from err
@@ -242,7 +250,7 @@ class API:  # pylint: disable=too-many-instance-attributes
                 },
             )
             token_resp.raise_for_status()
-        except ClientResponseError as err:
+        except Exception as err:  # pylint: disable-broad-except
             raise SimplipyError(
                 f"Unknown error while attempting getting access token: {err}"
             ) from err
@@ -253,7 +261,9 @@ class API:  # pylint: disable=too-many-instance-attributes
     async def _async_handle_on_backoff(self, _: dict[str, Any]) -> None:
         """Handle a backoff retry."""
         err_info = sys.exc_info()
-        err: ClientResponseError = err_info[1].with_traceback(err_info[2])  # type: ignore
+        err: ClientResponseError = err_info[1].with_traceback(  # type: ignore
+            err_info[2]
+        )
 
         LOGGER.debug("Error during request attempt: %s", err)
 
@@ -302,11 +312,15 @@ class API:  # pylint: disable=too-many-instance-attributes
             raise RequestError(
                 f"Request error while attempting to refresh access token: {err}"
             ) from err
+        except Exception as err:  # pylint: disable-broad-except
+            raise SimplipyError(
+                f"Error while attempting to refresh access token: {err}"
+            ) from err
 
         await self._async_save_token_data_from_response(token_resp)
 
         for callback in self._refresh_token_callbacks:
-            schedule_callback(callback, self.refresh_token)
+            execute_callback(callback, self.refresh_token)
 
     async def _async_api_request(
         self, method: str, endpoint: str, url_base: str = API_URL_BASE, **kwargs: Any
@@ -476,7 +490,7 @@ class API:  # pylint: disable=too-many-instance-attributes
             },
         )
 
-        LOGGER.info(
+        LOGGER.debug(
             "Login Verification Response Headers: %s", login_verification_resp.headers
         )
 
